<?php
/*
This file is part of Mailboy.

Mailboy is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Mailboy is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Mailboy.  If not, see <http://www.gnu.org/licenses/>.
*/

$yn_monoselect = "monoselect|Y=yes:N=no";

// Set up policy form *once*
$policy_form = new FormMaker("policy", genlink("page=policies"));
$policy_form->add_element("policy_name", "text", "free|127", 1);
$policy_form->add_element("virus_lover", "radio", $yn_monoselect, 1);
$policy_form->add_element("spam_lover", "radio", $yn_monoselect, 1);
$policy_form->add_element("banned_files_lover", "radio", $yn_monoselect, 1);
$policy_form->add_element("bad_header_lover", "radio", $yn_monoselect, 1);
$policy_form->add_element("bypass_virus_checks", "radio", $yn_monoselect, 1);
$policy_form->add_element("bypass_spam_checks", "radio", $yn_monoselect, 1);
$policy_form->add_element("bypass_banned_checks", "radio", $yn_monoselect, 1);
$policy_form->add_element("bypass_header_checks", "radio", $yn_monoselect, 1);
$policy_form->add_element("spam_quarantine_to", "text", "email");
$policy_form->add_element("spam_tag_level", "text", "integer", 1);
$policy_form->add_element("spam_tag2_level", "text", "integer", 1);
$policy_form->add_element("spam_kill_level", "text", "integer", 1);
$policy_form->add_element("spam_subject_tag2", "text", "free|64");

// Set some user friendly display names
$policy_form->set_display_name("spam_tag_level", "Add spam tags (score)");
$policy_form->set_display_name("spam_tag2_level", "Mark as spam (score)");
$policy_form->set_display_name("spam_kill_level", "Discard spam (score)");
$policy_form->set_display_name("spam_subject_tag2", "Spam subject line (empty for no change)");

// Add policies
if (am_i_server_admin(2)) {
	$policy_form->set_form_name("new_policy");
	if ($_POST['new_policy']) {
		$policy_form->check_post($_POST);
		if (!$policy_form->errors()) {
			$form_values = $policy_form->get_values();
			$q = $cnx->insert($form_values, "policy", 0);
			if (is_valid_id($q)) {
				$content['status'][] = "New policy added";
			} else {
				$content['errors'][] = "Error inserting in database";
				$content['errors'][] = $q;
			}
		} else {
			$content['errors'][] = "Form contains errors";
		}
	}
	$content['policies']['forms']['add_policy'] = $policy_form->result();
}

// Delete policies - use _GET
if (am_i_server_admin(4) AND is_valid_id($_GET['delete'])) {
	$del_ar = $cnx->select_row("id,policy_name", "policy", "WHERE `id` = ".$_GET['delete']."");
	if ($del_ar['id'] == $_GET['delete']) {
		if ($_GET['confirm']) {
			//delete
			$q = $cnx->delete($del_ar['id'], "policy", "id");
			if (is_valid_id($q)) {
				$content['status'][] = "Policy deleted";
			} else {
				$content['errors'][] = "Error deleting from database";
				$content['errors'][] = $q;
			}
		} else {
			$content['policies']['confirm_delete']['policy'] = $del_ar['policy_name'];
			$content['policies']['confirm_delete']['confirm_link'] = genlink("page=policies|delete=".$_GET['delete']."|confirm=1");
			$content['policies']['confirm_delete']['cancel_link'] = genlink("page=policies");
		}
	}
}

// Edit policies - use GET ??
if (am_i_server_admin(3) AND is_valid_id($_GET['edit'])) {
	$policy_data = $cnx->select_row("*", "policy", "WHERE `id` ='".$_GET['edit']."'");
	if ($policy_data['id'] == $_GET['edit']) {
		$policy_form->set_form_name("edit_policy");
		$policy_form->set_form_action(genlink("page=policies|edit=".$_GET['edit'].""));
		$policy_form->add_element("id", "hidden", "custom|#^".$_GET['edit']."$#", 1);
		if ($_POST['edit_policy']) {
			$policy_form->check_post($_POST);
			if (!$policy_form->errors()) {
				$form_values = $policy_form->get_values();
				$q = $cnx->update($form_values, "policy", $_GET['edit'], 0);
				if (is_valid_id($q)) {
					$content['status'][] = "Policy updated";
				} else {
					$content['errors'][] = "Error inserting in database";
					$content['errors'][] = $q;
				}
			} else {
				$content['errors'][] = "Form contains errors";
			}
		} else {
			// load values
			foreach ($policy_data as $k => $v) {
				if ($v) {
					$policy_form->set_value_and_validate($k, $v);
				}
			}
		}
		$content['policies']['forms']['edit_policy'] = $policy_form->result();
	} else {
		$content['errors'][] = "Policy does not exist";
	}
}

// List policies
$order_stuff = check_order_by('policies');
$order_by = $order_stuff['order_by'];
$content['policies']['list']['order_by'] = $order_stuff['th'];
$content['policies']['list']['policies'] = $cnx->select("*", "policy", $order_by);
foreach ($content['policies']['list']['policies'] as $k => $v) {
	if (am_i_server_admin(4)) {
		$content['policies']['list']['policies'][$k]['delete_link'] = genlink("page=policies|delete=".$v['id']."");
	}
	if (am_i_server_admin(3)) {
		$content['policies']['list']['policies'][$k]['details_link'] = genlink("page=policies|edit=".$v['id']."");
	}
}

?>
