#!/usr/bin/php
<?php
/*
This file is part of Mailboy.

Mailboy is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Mailboy is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Mailboy.  If not, see <http://www.gnu.org/licenses/>.

This script is forked from check_mysql.php by LISSY Alexandre, "lissyx" <alexandrelissy@free.fr>
Its purpose is to enable authentification by ejabberd to mailboy users

TODO add instructions
*/


define('ROOT', dirname(__FILE__)."/");

if (file_exists(ROOT . "db_config.php")) {
	include(ROOT . "db_config.php");
} else if (file_exists(ROOT . "../db_config.php")) {
	 include(ROOT . "../db_config.php");
} else if (file_exists(ROOT . "../config/db_config.php")) {
	 include(ROOT . "../config/db_config.php");
} else {
	exit();
}


$auth = new EjabberdAuth();

$auth->dbhost = $db_host;
$auth->dbuser = $db_user;
$auth->dbpass = $db_pass;
$auth->dbbase = $db_database;

$auth->play();

class EjabberdAuth {

	// For betterness, declare all vars
	var $c; // MySQL connection thingy

	// MySQL details
	var $dbhost;
	var $dbuser;
	var $dbpass;
	var $dbbase;

	var $data;	// Data received from ejabberd
	var $email;
	var $user;
	var $domain;
	var $pass;

	var $debug	= true;								// Debug mode
	var $debugfile	= "/var/log/ejabberd/mailboy_auth-debug.log";	// Debug output
	var $logging 	= true;								// Do we log requests ?
	var $logfile 	= "/var/log/ejabberd/mailboy_auth-log.log";		// Log file

	//var $dateformat = "M d H:i:s"; /* Check date() for string format. */
	//var $command; /* This is the command sent ... */
	var $stdin;   /* stdin file pointer */
	var $stdout;  /* stdout file pointer */

	// Functions I don't want to rewrite yet
	function __construct() { // previous JabberAuth
		// @define_syslog_variables(); DEPRECATED
		@openlog("mailboy-ext-auth", LOG_NDELAY, LOG_SYSLOG);

		if($this->debug) { // FIXME does this even lift ?
			@error_reporting(E_ALL);
			@ini_set("log_errors", "1");
			@ini_set("error_log", $this->debugfile);
		}
		$this->logg("Starting Mailboy Ejabberd external-auth ..."); // We notice that it's starting ...

		//$this->openstd(); // WHY a new function ?
		$this->stdout = @fopen("php://stdout", "w"); // We open STDOUT so we can read
		$this->stdin  = @fopen("php://stdin", "r"); // and STDIN so we can talk !
	}
	function play() {
		do {
			//$this->readstdin(); // get data
			// Get data
			$l      = @fgets($this->stdin, 3); // We take the length of string // What for ??
			$length = @unpack("n", $l); // ejabberd give us something to play with ...
			$len    = $length["1"]; // and we now know how long to read.
			if($len > 0) { // if not, we'll fill logfile ... and disk full is just funny once
				//$this->logg("Reading $len bytes ... "); // We notice ... // Who cares ?
				$data   = @fgets($this->stdin, $len+1);
				// $data = iconv("UTF-8", "ISO-8859-15", $data); // To be tested, not sure if still needed.
				$this->data = $data; // We set what we got.
				//$this->logg("IN: ".$data); // Don't log this, it contains the password in cleartext
			}

			$length = strlen($this->data); // compute data length
			//if($length > 0 ) { // for debug mainly ...
				//$this->logg("data length is : ".$length);
			//}

			$ret = $this->command(); // play with data !

			//$this->logg("RE: " . $ret); // this is what WE send. // It's true/false, so no need to log this
			$this->out($ret); // send what we reply.
			$this->data = NULL; // more clean. ...
		} while (true);
	}
	function out($message) { // Write to output (ejabberd)
		@fwrite($this->stdout, $message); // We reply ...
		// Not sure why we need the following
		//$dump = @unpack("nn", $message);
		//$dump = $dump["n"];
		//$this->logg("OUT: ". $dump);

	}
	function logg($message) {
		if($this->logging) {
			@syslog(LOG_INFO, $message);
		}
	}
	function stop() {
		$this->logg("Shutting down ...");
		closelog();
		@fclose($this->stdin);
		@fclose($this->stdout);
		exit(0);
	}


	function mysql() {
		if (!@mysqli_ping($this->c)) {
			$this->logg("MYSQL not connected.... connecting");
			$this->c = mysqli_connect($this->dbhost, $this->dbuser, $this->dbpass, $this->dbbase);
			if ($this->c) {
				return true;
			} else {
				$this->logg("MYSQL ERROR");
				return false;
			}
		} else {
			$this->logg("MYSQL already connected....");
			return true;
		}
	}

	function command() {
		// Pour data into array
		$data = explode(":", $this->data);
		//$this->cmd = $data[0];
		//$this->user = $data[1];
		//$this->domain = $data[2];
		//$this->pass = $data[3];
		//$this->email = $data[1]."@".$data[2];

		// Check MySQL connection. If down, open and come back
		if ($this->mysql()) {

			$this->logg("Running cmd: ".$data[0]."");

			switch($data[0]) {

				case "isuser": // this is the "isuser" command, used to check for user existance
						$this->email = mysqli_real_escape_string($this->c, $data[1]."@".$data[2]);
						$return = $this->checkuser(); // Check if this->email exists
					break;

				case "auth": // check login, password
						$this->user = $data[1];
						$this->domain = $data[2];
						// Fix password trimming
						$this->pass = trim($data[3]);
						$this->email = mysqli_real_escape_string($this->c, $data[1]."@".$data[2]);
						$return = $this->checkpass();
					break;

				case "setpass":
						$return = false; // We do not want jabber to be able to change password
					break;

				default:
						$this->stop(); // if it's not something known, we have to leave.
					break;
			}

			$return = ($return) ? 1 : 0;
			//if(strlen($data[0]) > 0 && strlen($parms) > 0) { // Meh.
				//$this->logg("CMD: ".$data[0].":".$parms." => ".$return." ");
			//}
			return @pack("nn", 2, $return);

		} else {
			$this->logg("MYSQL is broken. Please fix it");
			$this->stop();
		}
	}

	function checkpass() {
		// Get password hash
		$pass_q = mysqli_query($this->c, "SELECT password FROM `view_xmpp_users` WHERE `email` = '$this->email';");
		$pass_a = mysqli_fetch_assoc($pass_q);
		preg_match('#{(.*)}(.*)#',$pass_a['password'], $password_data); // Thanks konrados

		// Check if password matches - Retro compatible
		if ($password_data[1] == "PLAIN-MD5") {
			$this->logg("password scheme is PLAIN-MD5");
			if (md5($this->pass) == $password_data[2]) {
				$this->logg("$this->email: Login OK");
				return true;
			} else {
				$this->logg("$this->email: Login denied");
				return false;
			}
		} else {
			// Both password_verify AND hash_equals work with SHA512/256, anything salted....
			if (password_verify($this->pass, $password_data[2])) {
				$this->logg("$this->email: Login OK");
				return true;
			} else {
				$this->logg("$this->email: Login denied");
				return false;
			}
		}

	}

	function checkuser() {
		$user_q = mysqli_query($this->c, "SELECT 1 FROM `view_xmpp_users` WHERE `email` = '$this->email';");
		$user_a = mysqli_fetch_assoc($user_q);
		if ($user_a[1] == 1) {
			$this->logg("$this->email: User exists");
			return true;
		} else {
			$this->logg("$this->email: User does not exist");
			return false;
		}

	}
}
